/*
 * This file is generated by ui/scripts/api_codegen.mjs.
 * Do not edit this file directly.
 */

import type { components, paths } from './schema'

export type OpenAPIComponents = components
export type OpenAPIPaths = paths
export type CameraResponse = components["schemas"]["CameraResponse"]
export type CameraListResponse = CameraResponse[]
export type CameraCreate = components["schemas"]["CameraCreate"]
export type CameraUpdate = components["schemas"]["CameraUpdate"]
export type ConfigChangeResponse = components["schemas"]["ConfigChangeResponse"]
export type SetupStatusResponse = components["schemas"]["SetupStatusResponse"]
export type FinalizeRequest = components["schemas"]["FinalizeRequest"]
export type FinalizeResponse = components["schemas"]["FinalizeResponse"]
export type PreflightCheckResponse = components["schemas"]["PreflightCheckResponse"]
export type PreflightResponse = components["schemas"]["PreflightResponse"]
export type TestConnectionRequest = components["schemas"]["TestConnectionRequest"]
export type TestConnectionResponse = components["schemas"]["TestConnectionResponse"]
export type HealthResponse = components["schemas"]["HealthResponse"]
export type StatsResponse = components["schemas"]["StatsResponse"]
export type DiagnosticsResponse = components["schemas"]["DiagnosticsResponse"]
export type RuntimeReloadResponse = components["schemas"]["RuntimeReloadResponse"]
export type RuntimeState = components["schemas"]["RuntimeState"]
export type RuntimeStatusResponse = components["schemas"]["RuntimeStatusResponse"]
export type ClipListResponse = components["schemas"]["ClipListResponse"]
export type ClipResponse = components["schemas"]["ClipResponse"]
export type ClipStatus = components["schemas"]["ClipStatus"]
export type DiscoverRequest = components["schemas"]["DiscoverRequest"]
export type DiscoveredCameraResponse = components["schemas"]["DiscoveredCameraResponse"]
export type ProbeRequest = components["schemas"]["ProbeRequest"]
export type ProbeResponse = components["schemas"]["ProbeResponse"]
export type MediaProfileResponse = components["schemas"]["MediaProfileResponse"]
export type DeviceInfoResponse = components["schemas"]["DeviceInfoResponse"]
export type ListClipsQuery = NonNullable<paths["/api/v1/clips"]["get"]["parameters"]["query"]>
